/************************************************************************/
/*                                                                      */
/* Program: dragon2.c                                                   */
/*                                                                      */
/* Dragon Curve                                                         */
/* ============                                                         */
/*                                                                      */
/* This program calculates a Dragon curve.                              */
/* Image size: 1024 x 1024 pixels.                                      */
/*                                                                      */
/* Algorithm by:                                                        */
/*                                                                      */
/* Henning von Rosen and Stefan Spaennare, Lund, 1985.                  */
/*                                                                      */
/* Program by: Stefan Spaennare, Lund, Sweden                           */
/* E-mail:        stefan@spaennare.se                                   */
/*                                                                      */
/* First version: June 1995                                             */
/* Latest update: 1998-08-22                                            */
/*                                                                      */
/************************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include "image.h"

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;

void line(im,x1,y1,x2,y2,col)
short **im;
int x1,y1,x2,y2;
int col;
{
   double dx,dy,r,rr,ax,ay,x,y;
   int i,xi,yi,ri;

   dx=(double)(x2)-(double)(x1);
   dy=(double)(y2)-(double)(y1);

   r=sqrt(dx*dx+dy*dy);

   ri=2*(int)(r+1.0);

   rr=(double)(ri);

   ax=dx/rr;
   ay=dy/rr;

   x=(double)(x1);
   y=(double)(y1);

   for (i=0; i<=ri; i++) {
      xi=(int)(x+0.5);
      yi=(int)(y+0.5);
      im[yi][xi]=col;
      x=x+ax;
      y=y+ay;
   } /* for i */

} /* line */


int main(argc,argv)
int argc;
char *argv[];
{
   struct image_head header;    
   short **image;

   double flops,ftime;


   int x72,y72,x7,y7,i,n,k,v,v0,dd,nn1,nn2,q,nn,p,col;
   int size,j;

   double v1,v2,c,r,dt,x,y,z,x1,y1,x0,y0,ss;

   clock_t time1,timediff;


   if (argc<2) {
      printf("Usage: %s image\n",argv[0]);
      exit(0);
   } /* if */


   size=1024;

   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=size;
   header.wid=size;
 
   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);


   for (i=0; i<size; i++) {
      for (j=0; j<size; j++) {
         image[i][j]=32767;
      } /* for j */
   } /* for i */

   time1=clock();


   p=17;
   dd=1;

   nn=1;
   for (i=1; i <= p; i++) {
      nn=2*nn;
   } /* for i */

   r=1.0;
   c=2.0*r;
   x0=780.0;
   y0=340.0;
   v0=1;
   v=0;
   x=0.0;
   y=0.0;

   x72=(int)(x0+0.5);
   y72=(int)(y0+0.5);

   col=0;
   n=1;


  for (n=1; n<nn; n++) {

     k=1;

     do {
	nn1=(n-k) % (4*k);
	nn2=(n+k) % (4*k);
	k=2*k;
     } while ((nn1 != 0) && (nn2 != 0));

     if (nn2 == 0) {
	q=1;
     } /* if */

     if (nn1 == 0) {
	q=0;
     } /* if */


     if (v0 == 0) {
	if (v==1) {
	   if (q==0) {
	      x=x+c; v0=3; v=2;
	   }
	   else {
	      v0=1; v=2;
	   } /* if */
	}
	else {
	   if (q==0) {
	      v0=3; v=2;
	   }
	   else {
	      x=x-c; v0=1; v=2;
	   } /* if */
	} /* if */
     } 
     else if (v0 ==1) {
	if (v==0) {
	   if (q==0) {
	      v0=0; v=3;
	   }
	   else {
	      y=y-c; v0=2; v=3;
	   } /* if */
	}
	else {
	    if (q==0) {
	       y=y+c; v0=0; v=3;
	    }
	    else {
	       v0=2; v=3;
	    } /* if */
	} /* if */
     } 
     else if (v0 == 2) {
	if (v==1) {
	   if (q==0) {
	      v0=1; v=0;
	   }
	   else {
	      x=x+c; v0=3; v=0;
	   } /* if */
	}
	else {
	   if (q==0) {
	      x=x-c; v0=1; v=0;
	   }
	   else {
	      v0=3; v=0;
	   } /* if */
	} /* if */
     } 
     else if (v0 == 3) {
	if (v==0) {
	   if (q==0) {
	      y=y-c; v0=2; v=1;
	   }
	   else {
	      v0=0; v=1;
	   } /* if */
	}
	else {
	   if (q==0) {
	      v0=2; v=1;
	   }
	   else {
	      y=y+c; v0=0; v=1;
	   } /* if */
	} /* if */
     }  /* if */


     x7=(int)(x+x0+0.5);
     y7=(int)(y+y0+0.5);

     line(image,x7,y7,x72,y72,col);

     x72=x7;
     y72=y7;

  } /* for n */ 


timediff=clock()-time1;

ftime=(double)(timediff)/timetic;
printf("\nTime:  %12.2f\n",ftime);

WriteImage(argv[1],image,header,im_s16);

} /* End */
